/*
 *  ======== ti_ndk_config.c ========
 *  Configured module definitions for the NDK
 *
 *  DO NOT EDIT - This file is generated by the SysConfig tool.
 */


/*
 *  ======== ti.ndk/General ========
 */
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

#include <signal.h>
#include <time.h>


#include <ti/drivers/emac/EMACMSP432E4.h>
#include <ti/ndk/inc/netmain.h>
#include <ti/ndk/inc/stkmain.h>
#include <ti/ndk/inc/os/oskern.h>
#include <ti/ndk/os/NDK_SemaphoreP.h>

#include <FreeRTOS.h>
#include <task.h>
#include <semphr.h>
#include <queue.h>

/* forward declarations from other modules */
static void configTcp(void *hCfg);
static void configUdp(void *hCfg);
static void ti_ndk_config_ip_init(void *hCfg);

/* socket file descriptor table */
#define MAXSOCKETS 10
uint32_t ti_ndk_socket_max_fd = MAXSOCKETS;
void *ti_ndk_socket_fdtable[MAXSOCKETS];



/* NDK memory manager page size and number of pages [used by mmAlloc()] */
#define RAW_PAGE_SIZE 4096
#define RAW_PAGE_COUNT 10

const int ti_ndk_config_Global_rawPageSize  = RAW_PAGE_SIZE;
const int ti_ndk_config_Global_rawPageCount = RAW_PAGE_COUNT;

/* P.I.T. (page information table) */
#if defined(__ti__)
#pragma DATA_SECTION(ti_ndk_config_Global_pit, ".bss:NDK_MMBUFFER");
#pragma DATA_SECTION(ti_ndk_config_Global_pitBuffer, ".bss:NDK_MMBUFFER");
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#elif defined (__IAR_SYSTEMS_ICC__)
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#elif defined (__GNUC__)
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT] __attribute__ ((section(".bss:NDK_MMBUFFER")));
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT] __attribute__ ((section(".bss:NDK_MMBUFFER")));
#else
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#endif

/* Memory bucket sizes */
#define SMALLEST 48
#define LARGEST (RAW_PAGE_SIZE)

const int ti_ndk_config_Global_smallest = SMALLEST;
const int ti_ndk_config_Global_largest  = LARGEST;

/* Memory Slot Tracking */
uint32_t ti_ndk_config_Global_Id2Size[] =
        {SMALLEST, 96, 128, 256, 512, 1536, LARGEST};

/*
 *  Local Packet Buffer Pool Definitions
 *
 *  The below variables/defines are used to override the defaults that are set
 *  in the Packet Buffer Manager (PBM) file src/stack/pbm/pbm_data.c
 */

/*
 *  Number of buffers in PBM packet buffer free pool
 *
 *  The number of buffers in the free pool can have a significant effect
 *  on performance, especially in UDP packet loss. Increasing this number
 *  will increase the size of the static packet pool use for both sending
 *  and receiving packets.
 */
#define PKT_NUM_FRAMEBUF 20

/* Size of Ethernet frame buffer */
#define PKT_SIZE_FRAMEBUF 1536

const int ti_ndk_config_Global_numFrameBuf = PKT_NUM_FRAMEBUF;
const int ti_ndk_config_Global_sizeFrameBuf = PKT_SIZE_FRAMEBUF;

/* Data space for packet buffers */
unsigned char ti_ndk_config_Global_pBufMem[PKT_NUM_FRAMEBUF * PKT_SIZE_FRAMEBUF] __attribute__ ((aligned(4), section(".bss:NDK_PACKETMEM")));
unsigned char ti_ndk_config_Global_pHdrMem[PKT_NUM_FRAMEBUF * sizeof(PBM_Pkt)] __attribute__ ((aligned(4), section(".bss:NDK_PACKETMEM")));

static char *hostname = "tisoc";

extern void llTimerTick();

/*
 *  ======== configTcp ========
 *  Configure the TCP settings
 *
 *  TCP configuration default values
 *
 *  Value   Configuration Property
 *  ----------------------------------------
 *  8192    TCP transmit buffer size
 *  8192    TCP receive buffer size (copy mode)
 *  8192    TCP receive size maximum (non-copy mode)
 *
 *  [1] Configuration entries are not added when the
 *      property is configured to its default value.
 *  [2] Default values are device specific.
 */
static void configTcp(void *hCfg)
{
    uint32_t tcpTxBufSize = 1024;
    uint32_t tcpRxBufSize = 1024;
    uint32_t tcpRxBufLimit = 2048;

    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPTXBUF, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpTxBufSize, NULL);
    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPRXBUF, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufSize, NULL);
    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPRXLIMIT, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufLimit, NULL);
}

/*
 *  ======== configUdp ========
 *  Configure the UDP settings
 */
static void configUdp(void *hCfg)
{
    uint32_t tcpRxBufSize = 4096;

    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKUDPRXLIMIT, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufSize, NULL);
}

/*
 *  ======== ndkStackThread ========
 *  NDK main thread function
 */
static void ndkStackThread(void *argUnused)
{
    extern void netIPAddrHook();
    void *hCfg;
    uint32_t u32cval;
    int rc;
    timer_t ndkHeartBeat;
    struct sigevent sev;
    struct itimerspec its;
    struct itimerspec oldIts;
    int ndkHeartBeatCount = 0;

    /* create the NDK timer tick */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_value.sival_ptr = &ndkHeartBeatCount;
    sev.sigev_notify_attributes = NULL;
    sev.sigev_notify_function = &llTimerTick;

    rc = timer_create(CLOCK_MONOTONIC, &sev, &ndkHeartBeat);
    if (rc != 0) {
        /* ndkStackThread: failed to create timer */
    }

    /* start the NDK 100ms timer */
    its.it_interval.tv_sec = 0;
    its.it_interval.tv_nsec = 100000000;
    its.it_value.tv_sec = 0;
    its.it_value.tv_nsec = 100000000;

    rc = timer_settime(ndkHeartBeat, 0, &its, NULL);
    if (rc != 0) {
        /* ndkStackThread: failed to set time */
    }

    /* user begin hook == null */

    rc = NC_SystemOpen(NC_PRIORITY_LOW, NC_OPMODE_INTERRUPT);
    if (rc) {
        /* ndkStackThread: NC_SystemOpen Failed */
        goto nc_fail;
    }

    /* create and build the system configuration from scratch */
    hCfg = CfgNew();
    if (!hCfg) {
        /* ndkStackThread: Unable to create configuration */
        goto main_exit;
    }

    /* user init hook == null */

    /* Add hostname */
    CfgAddEntry(hCfg, CFGTAG_SYSINFO, CFGITEM_DHCP_HOSTNAME, 0,
                strlen(hostname), (unsigned char *)hostname, NULL);

    /* TCP and UDP config */
    configTcp(hCfg);
    configUdp(hCfg);

    /* IP configuration */
    ti_ndk_config_ip_init(hCfg);

    /* config low priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKLOW, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /* config normal priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKNORM, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /* config high priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKHIGH, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /*  Boot the system using this configuration
     *
     *  Loop until the function returns 0. This facilitates a reboot command.
     */
    do
    {
        rc = NC_NetStart(hCfg, NULL, NULL, netIPAddrHook);

        /* user reboot hook == null */

    } while (rc > 0);

    /* user delete hook == null */

    /* shut down the stack */
    CfgFree(hCfg);

main_exit:
    NC_SystemClose();

nc_fail:
    /* stop and delete the NDK heartbeat */
    its.it_value.tv_sec = 0;
    its.it_value.tv_nsec = 0;
    rc = timer_settime(ndkHeartBeat, 0, &its, &oldIts);
    rc = timer_delete(ndkHeartBeat);

    /* ndkStackThread: exiting ... */
}

/*
 * ======== ti_ndk_config_Global_startupFxn ========
 * Called to start up the NDK. In BIOS, this can be called as a BIOS startup
 * function, or from main(). In FreeRTOS, this should be called from main().
 */
void ti_ndk_config_Global_startupFxn()
{
    BaseType_t status;
    TaskHandle_t ndkThread = NULL;

    status = xTaskCreate((TaskFunction_t)ndkStackThread, "ndkStackThread",
            2048 / sizeof(portSTACK_TYPE), NULL, 5, &ndkThread);

    if (status != pdPASS) {
        /* Error: could not create NDK stack thread */
        while(1);
    }
}


/*
 *  ======== ti.ndk/NIMU ========
 */
NIMU_DEVICE_TABLE_ENTRY NIMUDeviceTable[2] = {
    {
        .init = EMACMSP432E4_NIMUInit
    },
    {NULL}
};

/*
 *  ======== ti_ndk_config_ip_init ========
 *  Configure the IP settings
 */
static void ti_ndk_config_ip_init(void *hCfg)
{
    CI_SERVICE_DHCPC dhcpc;
    static unsigned char DHCP_OPTIONS[] = { DHCPOPT_SUBNET_MASK };

    /* ----------------- Configuring IP for CONFIG_NIMU_0 ----------------- */
    /* NOTE!  The application must provide the service report function
     * matching this prototype!
     */
    extern void serviceReport(uint32_t item, uint32_t status,
            uint32_t report, void *h);


    /* use DHCP to obtain IP address on interface  */
    memset(&dhcpc, 0, sizeof(dhcpc));
    /*
     * CIS_FLG_IFIDXVALID makes this DHCP Client bind to the ip address of this
     * NIMU interface, 1. This is the only appropriate mode for the DHCP
     * Client.
     */
    dhcpc.cisargs.Mode = CIS_FLG_IFIDXVALID;
    dhcpc.cisargs.IfIdx = 1;
    dhcpc.cisargs.pCbSrv = serviceReport;
    dhcpc.param.pOptions = DHCP_OPTIONS;
    dhcpc.param.len = 1;
    CfgAddEntry(hCfg, CFGTAG_SERVICE, CFGITEM_SERVICE_DHCPCLIENT, 0,
            sizeof(dhcpc), (unsigned char *)&dhcpc, NULL);

}


/*
 *  ======== ti.ndk/OSAL ========
 */

/* Compute number of microseconds per tick based on user's kernel settings */
#define NDK_FREERTOS_USEC_PER_TICK ((1000000) / (configTICK_RATE_HZ))

static bool hookOK = false;

/*
 * ======== _TaskInit ========
 * Called from NETCTRL init
 */
void _TaskInit()
{
    hookOK = true;
}

/*
 * ======== _TaskShutdown ========
 * Called from NETCTRL shutdown
 */
void _TaskShutdown()
{
    hookOK = false;
}

/*
 * ======== TaskBlock ========
 * Block a task
 */
void TaskBlock(void *h)
{
    vTaskSuspend(NULL);
}

/*
 * ======== ti_ndk_os_threadStub ========
 * Wrapper function used by TaskCreate(). This function conforms to the
 * FreeRTOS Task function signature:
 *
 *     void vFunction(void *pvParameters)
 *
 * It is needed in order to pass the Task arguments that were passed in the
 * TaskCreate() call into the user's actual Task function. It also ensures the
 * Task exits properly, as required by FreeRTOS.
 */
static void ti_ndk_os_threadStub(void *threadArgs)
{
    ti_ndk_os_TaskArgs *args = (ti_ndk_os_TaskArgs *)threadArgs;

    /*
     * Copy args out of TaskArgs object into our Task's stack. This way we can
     * free args before the user thread function is called.
     */
    void (*taskFunction)() = args->arg0;
    uintptr_t argument1 = args->arg1;
    uintptr_t argument2 = args->arg2;

    free(args);

    /* open the file descriptor session for the user here */
    fdOpenSession(TaskSelf());

    /* call user thread function */
    taskFunction(argument1, argument2);

    /* close the file descriptor session for the user here */
    fdCloseSession(TaskSelf());

    /* Exit task. FreeRTOS Tasks cannot simply fall out of their functions: */
    TaskExit();
}

/*
 * ======== TaskCreate ========
 * Create a task
 */
void *TaskCreate(void (*pFun)(), char *Name, int Priority, uint32_t StackSize,
        uintptr_t Arg1, uintptr_t Arg2, uintptr_t argReserved)
{
    BaseType_t status;
    TaskHandle_t task = NULL;
    ti_ndk_os_TaskArgs *taskArgs;

    taskArgs = malloc(sizeof(ti_ndk_os_TaskArgs));
    if (!taskArgs) {
        /* Error: couldn't alloc task function args */
        return (NULL);
    }
    taskArgs->arg0 = pFun;
    taskArgs->arg1 = Arg1;
    taskArgs->arg2 = Arg2;

    /* xTaskCreate requires Name to not be NULL */
    if (Name == NULL) {
        Name = "ndkThread";
    }

    status = xTaskCreate((TaskFunction_t)ti_ndk_os_threadStub, Name,
            StackSize / sizeof(portSTACK_TYPE), (void *)taskArgs,
            (UBaseType_t)Priority, &task);

    if (status != pdPASS) {
        free(taskArgs);
        return (NULL);
    }

    return ((void *)task);
}

/*
 * ======== TaskDestroy ========
 * Destroy a task
 */
void TaskDestroy(void *h)
{
    if (h == TaskSelf()) {
        vTaskDelete(NULL);
    }
    else {
        vTaskDelete((TaskHandle_t)h);
    }
}

/*
 * ======== TaskSetEnv ========
 * Sets the task's Environment pointer
 */
void TaskSetEnv(void *h, int Slot, void *hEnv)
{
    if (Slot) {
        return;
    }

    if (!hookOK) {
        /*
         * DbgPrintf() will not output this message for FreeRTOS currently.
         * However, it will still shut down the stack if the stack has been
         * configured to do so for the DBG_ERROR level
         */
        DbgPrintf(DBG_ERROR,"TaskSetEnv: FATAL: NDK not initialized");
        return;
    }

    vTaskSetThreadLocalStoragePointer((TaskHandle_t)h, NDK_TLS_INDEX, hEnv);
}

/*
 * ======== TaskGetEnv ========
 * Gets the task's Environment pointer
 */
void *TaskGetEnv(void *h, int Slot)
{
    if (Slot) {
        return (0);
    }

    if (!hookOK) {
        /*
         * DbgPrintf() will not output this message for FreeRTOS currently.
         * However, it will still shut down the stack if the stack has been
         * configured to do so for the DBG_ERROR level
         */
        DbgPrintf(DBG_ERROR,"TaskGetEnv: FATAL: NDK not initialized");
        return (0);
    }

    return ((void *)pvTaskGetThreadLocalStoragePointer((TaskHandle_t)h,
            NDK_TLS_INDEX));
}

/*
 * ======== TaskExit ========
 * Exits and destroys a task
 */
void TaskExit()
{
    /*
     * FreeRTOS tasks must "self delete." They cannot fall out of the
     * Task function without this code, otherwise a run time error will occur.
     */
    vTaskDelete(NULL);
}

/*
 * ======== TaskGetPri ========
 * Get a task's priority
 */
int TaskGetPri(void *h)
{
    return ((int)uxTaskPriorityGet((TaskHandle_t)h));
}

/*
 * ======== TaskSetPri ========
 * Set a task's priority
 */
int TaskSetPri(void *h, int priority)
{
    int oldPri = TaskGetPri(h);
    vTaskPrioritySet((TaskHandle_t)h, (UBaseType_t)priority);
    return (oldPri);
}

/*
 * ======== TaskSelf ========
 * Return handle of task itself
 */
void *TaskSelf()
{
    return ((void *)xTaskGetCurrentTaskHandle());
}

/*
 * ======== TaskSleep ========
 * Put a task into sleep
 */
void TaskSleep(uint32_t delay)
{
    uint32_t secs;
    uint32_t remMs;
    uint32_t remUs;
    uint32_t ticks;
    uint32_t remTicks;

    secs = delay / 1000; /* Whole number of seconds in delay */
    remMs = delay - (secs * 1000); /* Remainder of milliseconds in delay */
    remUs = remMs * 1000;

    ticks = secs * configTICK_RATE_HZ;
    remTicks =
        (remUs + NDK_FREERTOS_USEC_PER_TICK - 1) / NDK_FREERTOS_USEC_PER_TICK;

    ticks += remTicks;

    vTaskDelay((TickType_t)ticks);
}

/*
 * ======== TaskYield ========
 * Yield task
 */
void TaskYield()
{
     taskYIELD();
}

/*
 *  Maximum count for a semaphore.
 */
#define MAXCOUNT 0xffff

#define portVECTACTIVE_MASK  (0xFFUL)

static bool inISR(void)
{
    bool stat;

    if ((portNVIC_INT_CTRL_REG & portVECTACTIVE_MASK) == 0) {
        /* Not currently in an ISR */
        stat = false;
    }
    else {
        stat = true;
    }

    return (stat);
}

uintptr_t NDK_HwiP_disable(void)
{
    uintptr_t key;

    /*
     *  If we're not in ISR context, use the FreeRTOS macro, since
     *  it handles nesting.
     */
    if ((portNVIC_INT_CTRL_REG & portVECTACTIVE_MASK) == 0) {
        /* Cannot be called from an ISR! */
        portENTER_CRITICAL();
        key = 0;
    }
    else {
#ifdef __TI_COMPILER_VERSION__
        key = _set_interrupt_priority(configMAX_SYSCALL_INTERRUPT_PRIORITY);
#else
#if defined(__IAR_SYSTEMS_ICC__)
        asm volatile (
#else /* !__IAR_SYSTEMS_ICC__ */
            __asm__ __volatile__ (
#endif
                "mrs %0, basepri\n\t"
                "msr basepri, %1"
                : "=&r" (key)
                : "r" (configMAX_SYSCALL_INTERRUPT_PRIORITY)
                );
#endif
    }

    return (key);
}

void NDK_HwiP_restore(uintptr_t key)
{
    if ((portNVIC_INT_CTRL_REG & portVECTACTIVE_MASK) == 0) {
        /* Cannot be called from an ISR! */
        portEXIT_CRITICAL();
    }
    else {
#ifdef __TI_COMPILER_VERSION__
        _set_interrupt_priority(key);
#else
#if defined(__IAR_SYSTEMS_ICC__)
        asm volatile (
#else /* !__IAR_SYSTEMS_ICC__ */
            __asm__ __volatile__ (
#endif
                "msr basepri, %0"
                :: "r" (key)
                );
#endif
    }
}

#if (configUSE_COUNTING_SEMAPHORES == 0)
#error The NDK requires configUSE_COUNTING_SEMAPHORES be 1 in FreeRTOSConfig.h
#endif

NDK_SemaphoreP_Handle NDK_SemaphoreP_create(unsigned int count)
{
    SemaphoreHandle_t sem = NULL;

    /*
     *  The size of the semaphore queue is not dependent on MAXCOUNT.
     *
     *  FreeRTOS xSemaphoreCreateCounting() appears from the
     *  code in xQueueCreateCountingSemaphore() to create
     *  a queue of length maxCount, where maxCount is the
     *  maximum count that the semaphore should ever reach.
     *  However, the queue item size (queueSEMAPHORE_QUEUE_ITEM_LENGTH),
     *  is 0, so no actual memory is allocated for the queue items.
     *  Therefore we can pass any non-zero number as the maximum
     *  semaphore count.
     */
    sem = xSemaphoreCreateCounting((UBaseType_t)MAXCOUNT,
            (UBaseType_t)count);

    return ((NDK_SemaphoreP_Handle)sem);
}

NDK_SemaphoreP_Handle NDK_SemaphoreP_createBinary(unsigned int count)
{
    SemaphoreHandle_t sem = NULL;

    sem = xSemaphoreCreateBinary();
    if ((sem != NULL) && (count != 0)) {
        xSemaphoreGive(sem);
    }

    return ((NDK_SemaphoreP_Handle)sem);
}

void NDK_SemaphoreP_delete(NDK_SemaphoreP_Handle handle)
{
    vSemaphoreDelete((SemaphoreHandle_t)handle);
}

NDK_SemaphoreP_Status NDK_SemaphoreP_pend(NDK_SemaphoreP_Handle handle,
        uint32_t timeout)
{
    BaseType_t status;

    status = xSemaphoreTake((SemaphoreHandle_t)handle, timeout);

    if (status == pdTRUE) {
        return (NDK_SemaphoreP_OK);
    }

    return (NDK_SemaphoreP_TIMEOUT);
}

void NDK_SemaphoreP_post(NDK_SemaphoreP_Handle handle)
{
    BaseType_t xHigherPriorityTaskWoken;

    if (!inISR()) {
        xSemaphoreGive((SemaphoreHandle_t)handle);
    }
    else {
        xSemaphoreGiveFromISR((SemaphoreHandle_t)handle,
                &xHigherPriorityTaskWoken);
    }
}

